/*
 *  Jomar3D, a lobster-running-to-catch game.
 *  Copyright (C) 2006-2008 Jonathan Michalon <studios.chalmion@no-log.org>.
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#ifndef DRAW_H
#define DRAW_H

#include <GL/gl.h>
#include <GL/glu.h>
#include "core.h"


/* quality values */
enum
{
  Q_FINE      = 0,
  Q_NORMAL    = 1,
  Q_HORRIBLE  = 2
};

/* List number enum */
enum
{
  RESERVED,
  PINCER, ARM, ANT, TAIL, BODY, LEG,   /* lobster */
  HEAD, SOCCER, FIRE,                  /* martian */
  TOPBOT, WALLMAX, SMALLMAP, CLOUDS,   /* misc */
  MENU, CYL1, CYL2, CYL3, CYL4, CYL5, CYL6, CYL7, CYL8, CYL9, CYL10  /* MENU */
};

/* textures positions IDs in texture ID array */
enum
{
  TEXID_GROUND, /* ground */
  TEXID_EXT,    /* exterior wall */
  /* the wells poitions IDs must be aligned */
  TEXID_WALL1,  /* wall 1... */
  TEXID_WALL2,
  TEXID_WALL3,
  TEXID_WALL4,
  TEXID_WALL5,
  TEXID_WALL6,
  TEXID_WALL7,
  TEXID_WALL8,
  TEXID_WALL9,
  TEXID_WALL10, /* ... to wall 10 */
  TEXID_CLOUD,  /* clouds */
  TEXID_LOGO,   /* startup logo */
  TEXID_MENU,   /* menu background */
  
  TEXID_N_ID    /* # of textures, must be last */
};

/* flag whether the drawing of the Martian should have boosters on or off */
enum
{
  OFF    = 1<<0,
  ON     = 1<<1,
  SHADOW = 1<<2
};

/* Definition of a bomb */
typedef struct
{
  Coord2i   pos;      /* position (squares) */
  struct {
    float x;
    float z;
  }         angle;      /* drawing angles */
  int       throwtime;  /* date at which the bomb was launched */
  int       duration;   /* duration of the currently launched bomb (millisecond) */
} Bomb;


int         draw_getTexID           (int name);
GLUquadric* draw_getQuadric         (void);
int         draw_getQuality         (void);
int         draw_getVertMult        (void);
int         draw_postIntro          (void);
void        draw_adjustVars         (int elapsed,
                                     int action,
                                     Lobster *lobster);
void        draw_calculateFPS       (int flag);
void        draw_applyQuality       (int action,
                                     int value);
void        draw_init               (void);
GLuint*     draw_pickingRender      (int x,
                                     int y);
void        draw_redraw             (void);
void        draw_setGameAttributes  (void);
void        draw_setMusicName       (const char *text);
void        draw_loadTextures       (int first);
int         draw_isVisible          (int x,
                                     int y,
                                     float border);
void        draw_startBombDraw      (int posx,
                                     int posy,
                                     int duration);
void        draw_quit               (void);
void        draw_forceRedraw        (void);
void        draw_setView            (int which);
void        draw_moveViewH          (float dx,
                                     float dy);
void        draw_moveViewV          (float dx,
                                     float dy);
void        draw_shade              (float dx,
                                     float dy);
void        draw_rotateView         (float delta);



#endif /* guard */
