/*
 *  
 *  Copyright © 2008 Colomban "Ban" Wendling <ban@herbesfolles.org>
 *  
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */

/*
 * Creation date: Fri Aug  1 16:09:05 2008
 * Revision date: Fri Aug  1 16:09:05 2008
 */


#ifndef UTF8_H
#define UTF8_H

#ifdef __cplusplus
extern "C"
{
#endif


/* We only need that char has at least 8 bits, more is not a problem.
 * ISO-C99 standard says about CHAR_BIT:
 * "[its] implementation-defined value shall be equal or greater in magnitude
 *  (absolute value) to [8], with the same sign." */
/*
#include <limits.h>

#if CHAR_BIT < 8
# warning "CHAR_BIT is less than 8. You may experience problems"
#endif
*/


/* Unicode table offset (must be 0, only useful for readability) */
#define U 0

/* FIXME enum are written to int? too short for a full table */
/* Unicode table */
enum
{
  /* US-ASCII code, 1 byte
   * C0 controls */
  U_NUL  = U+0x0000,
  U_NULL = U+0x0000,
  U_SOH,
  U_STX,
  U_ETX,
  U_EOT,
  U_ENQ,
  U_ACK          = U+0x0006,
  U_ACKNOWLEDGE  = U+0x0006,
  U_BEL          = U+0x0007,
  U_BELL         = U+0x0007,
  U_BS           = U+0x0008,
  U_BACKSPACE    = U+0x0008,
  U_HT,
  U_LF,
  U_VT,
  U_FF,
  U_CR,
  U_SO,
  U_SI,
  U_DLE,
  U_DC1,
  U_DC2,
  U_DC3,
  U_DC4,
  U_NAK,
  U_SYN,
  U_ETB,
  U_CAN,
  U_EM,
  U_SUB,
  U_ESC,
  U_FS,
  U_GS,
  U_RS,
  U_US,
  /* printable chars */
  U_SP     = U+0x0020 /* 32 */,
  U_SPACE  = U+0x0020,
  U_EXCLAMATION_MARK = U+0x0021,
  U_EXCLAMATION      = U+0x0021,
  U_QUOTE,
  U_NUMBER_SIGN  = U+0x0023,
  U_SHARP        = U+0x0023,
  U_DOLLAR,
  U_PERCENT,
  U_AMPERSAND,
  U_APOSTROPHE,
  U_LEFT_BRACKET     = U+0x0028,
  U_LEFT_PARENTHESE  = U+0x0028,
  U_RIGHT_BRACKET    = U+0x0029,
  U_RIGHT_PARENTHESE = U+0x0029,
  U_ASTERISK,
  U_PLUS,
  U_COMMA,
  U_HYPEN,
  U_FULL_STOP  = U+0x002e /* 46 */,
  U_PERIOD     = U+0x002e,
  U_SLASH,
  U_0,
  U_1,
  U_2,
  U_3,
  U_4,
  U_5,
  U_6,
  U_7,
  U_8,
  U_9,
  U_COLON,
  U_SEMICOLON,
  U_LESS_THAN    = U+0x003c /* 60 */,
  U_LEFT_CHEVRON = U+0x003c,
  U_EQUAL,
  U_GREATER_THAN   = U+0x003e,
  U_RIGHT_CHEVRON  = U+0x003e,
  U_QUESTION_MARK  = U+0x003f,
  U_QUESTION       = U+0x003f,
  U_AT,
  U_A  = U+0x0041 /* 65 */,
  U_B,
  U_C,
  U_D,
  U_E,
  U_F,
  U_G,
  U_H,
  U_I,
  U_J,
  U_K,
  U_L,
  U_M,
  U_N,
  U_O,
  U_P,
  U_Q,
  U_R,
  U_S,
  U_T,
  U_U,
  U_V,
  U_W,
  U_X,
  U_Y,
  U_Z,
  U_LEFT_BOXBRACKET,
  U_BACKSLASH,
  U_RIGHT_BOXBRACKET,
  U_CARET,
  U_UNDERSCORE,
  U_GRAVE_ACCENT,
  U_a  = U+0x0061 /* 97 */,
  U_b,
  U_c,
  U_d,
  U_e,
  U_f,
  U_g,
  U_h,
  U_i,
  U_j,
  U_k,
  U_l,
  U_m,
  U_n,
  U_o,
  U_p,
  U_q,
  U_r,
  U_s,
  U_t,
  U_u,
  U_v,
  U_w,
  U_x,
  U_y,
  U_z,
  U_LEFT_BRACE,
  U_VERTICAL_BAR   = U+0x007c /* 124 */,
  U_PIPE           = U+0x007c,
  U_RIGHT_BRACE,
  U_TILDE,
  U_DEL,
  /* C1 controls */
  U__CONTROL_1,
  U__CONTROL_2,
  U_BREAK_PERMITTED_HERE = U+0x0082,
  U_BREAK_PERMITTED      = U+0x0082,
  U_NO_BREAK_HERE  = U+0x0083,
  U_NO_BREAK       = U+0x0083,
  U_INDEX,
  U_NEXT_LINE      = U+0x0085,
  U_NEL            = U+0x0085,
  U_START_OF_SELECTED_AREA,
  U_END_OF_SELECTED_AREA,
  U_CHARACTER_TABULTAION_SET = U+0x0008,
  U_HTS                      = U+0x0008, /* Horizontal Tabulation Set */
  U_CHARACTER_TABULATION_JUSTIFIED,
  U_LINE_TABULATION_SET  = U+0x008a,
  U_VTS                  = U+0x008a, /* Vertical Tabulation Set */
  U_PARTIAL_LINE_FORWARD,
  U_PARTIAL_LINE_BACKWARD,
  U_REVERSE_LINE_FEED  = U+0x008d,
  U_REVERSE_LF         = U+0x008d,
  U_SINGLE_SHIFT_2,
  U_SINGLE_SHIFT_3,
  U_DEVICE_CONTROL_STRING,
  U_PRIVATE_1,
  U_PRIVATE_2,
  U_SET_TRANSMIT_STATE,
  U_CANCEL, /* CANCEL_CHARACTER = 0x0094 */
  U_MESSAGE_WAITING,
  U_START_OF_GUARDED_AREA,
  U_END_OF_GUARDED_AREA,
  U_START_OF_STRING,
  U__CONTROL_3,
  U_SINGLE_CHARACTER_INTRODUCER  = U+0x009a,
  U_CONTROL_SEQUENCE_INTRODUCER,
  U_STRING_TERMINATOR,
  U_OPERATING_SYSTEM_COMMAND = U+0x009d,
  U_OSC                      = U+0x009d,
  U_PIVACY_MESSAGE,
  U_APPLICATION_PROGRAM_COMMAND  = U+0x009f,
  U_APC                          = U+0x009f,
  /* Latin-1 punctuation and symbols */
  U_NOBREAK_SPACE  = U+0x00a0,
  U_NBSP           = U+0x00a0,
  U_INVERTED_EXCLAMATION_MARK  = U+0x00a1,
  U_INVERTED_EXCLAMATION       = U+0x00a1,
  U_CENT_SIGN      = U+0x00a2,
  U_CENT           = U+0x00a2,
  U_POUND_SIGN     = U+0x00a3,
  U_POUND          = U+0x00a3,
  U_POUND_STRELING = U+0x00a3,
  U_IRISH_PUNT     = U+0x00a3,
  U_ITALIAN_LIRA   = U+0x00a3,
  U_TURKISH_LIRA   = U+0x00a3,
  U_CURRENCY_SIGN  = U+0x00a4,
  U_CURRENCY       = U+0x00a4,
  U_YEN_SIGN       = U+0x00a5,
  U_YEN            = U+0x00a5,
  U_YUAN_SIGN      = U+0x00a5,
  U_YUAN           = U+0x00a5,
  U_BROKEN_BAR           = U+0x00a6,
  U_BROKEN_VERTICAL_BAR  = U+0x00a6,
  U_PARTED_RULE          = U+0x00a6,
  U_SECTION_SIGN   = U+0x00a7,
  U_SECTION        = U+0x00a7,
  U_DIAERESIS,
  U_COPYRIGHT_SIGN = U+0x00a9,
  U_COPYRIGHT      = U+0x00a9,
  U_FEMININE_ORDINAL_INDICATOR,
  U_LEFT_GUILLEMET = U+0x00ab,
  U_LEFT_CHEVRONS  = U+0x00ab,
  U_NOT_SIGN       = U+0x00ac,
  U_NOT            = U+0x00ac,
  U_ANGLE_DASH     = U+0x00ac,
  U_SOFTH_HYPEN          = U+0x00ad,
  U_SHY                  = U+0x00ad,
  U_DISCRETIONARY_HYPEN  = U+0x00ad,
  U_REGISTERED_SIGN        = U+0x00ae,
  U_REGISTERED             = U+0x00ae,
  U_REGISTERED_TRADE_MARK  = U+0x00ae,
  U_MACRON         = U+0x00af,
  U_OVERLINE       = U+0x00af,
  U_DEGREE         = U+0x00b0,
  U_DEGREE_SIGN    = U+0x00b0,
  U_PLUSMINUS      = U+0x00b1,
  U_PLUSMINUS_SIGN = U+0x00b1,
  U_SUPERSCRIPT_2  = U+0x00b2,
  U_SQUARED        = U+0x00b2,
  U_SUPERSCRIPT_3  = U+0x00b3,
  U_CUBED          = U+0x00b3,
  U_ACUTE          = U+0x00b4,
  U_ACUTE_ACCENT   = U+0x00b4,
  U_mu             = U+0x00b5,
  U_Mu             = U+0x00b5,
  U_PILCROW        = U+0x00b6,
  U_PARAGRAPH_SIGN = U+0x00b6,
  U_ALINEA         = U+0x00b6,
  U_MIDDLE_DOT     = U+0x00b7,
  U_MIDPOINT       = U+0x00b7,
  U_CENTERED_DOT   = U+0x00b7,
  U_INTERPUNCT     = U+0x00b7,
  U_CEDILLA,
  U_SUPERSCRIPT_1,
  U_MASCULINE_ORDINAL_INDICATOR,
  U_RIGHT_GUILLEMET  = U+0x00bb,
  U_RIGHT_CHEVRONS   = U+0x00bb,
  U_VULGAR_FRACTION_ONE_QUARTER  = U+0x00bc,
  U_ONE_QUARTER                  = U+0x00bc,
  U_VULGAR_FRACTION_ONE_HALF = U+0x00bd,
  U_ONE_HALF                 = U+0x00bd,
  U_VULGAR_FRACTION_THREE_QUARTERS = U+0x00be,
  U_THREE_QUARTERS                 = U+0x00be,
  U_INVERTED_QUESTION_MARK = U+0x00bf,
  U_INVERTED_QUESTION      = U+0x00bf,
  /* Letters */
  U_A_GRAVE        = U+0x00c0,
  U_A_ACUTE,
  U_A_CIRCUMFLEX,
  U_A_TILDE,
  U_A_DIAERESIS    = U+0x00c4,
  U_A_TREMA        = U+0x00c4,
  U_A_RING,
  U_AE             = U+0x00c6,
  U_C_CEDILLA      = U+0x00c7,
  U_E_GRAVE        = U+0x00c8,
  U_E_ACUTE,
  U_E_CIRCUMFLEX,
  U_E_DIAERESIS    = U+0x00cb,
  U_E_TREMA        = U+0x00cb,
  U_I_GRAVE        = U+0x00cc,
  U_I_ACUTE,
  U_I_CIRCUMFLEX,
  U_I_DIAERESIS    = U+0x00cf,
  U_I_TREMA        = U+0x00cf,
  U_ETH            = U+0x00d0,
  U_N_TILDE,
  U_O_GRAVE        = U+0x00d2,
  U_O_ACUTE,
  U_O_CIRCUMFLEX,
  U_O_TILDE,
  U_O_DIAERESIS    = U+0x00d6,
  U_O_TREMA        = U+0x00d6,
  /* Mathematical operator */
  U_MULTIPLICATION_SIGN  = U+0x00d7,
  /* Letters (suit) */
  U_O_SLASHED      = U+0x00d8,
  U_U_GRAVE        = U+0x00d9,
  U_U_ACUTE,
  U_U_CIRCUMFLEX,
  U_U_DIAERESIS    = U+0x00dc,
  U_U_TREMA        = U+0x00dc,
  U_Y_ACUTE        = U+0x00dd,
  U_THORN          = U+0x00de,
  U_eszett         = U+0x00df,
  U_s_SHARP        = U+0x00df,
  /* Small letters */
  U_a_GRAVE        = U+0x00e0,
  U_a_ACUTE,
  U_a_CIRCUMFLEX,
  U_a_TILDE,
  U_a_DIAERESIS    = U+0x00e4,
  U_a_TREMA        = U+0x00e4,
  U_a_RING,
  U_ae             = U+0x00e6,
  U_c_CEDILLA      = U+0x00e7,
  U_e_GRAVE        = U+0x00e8,
  U_e_ACUTE,
  U_e_CIRCUMFLEX,
  U_e_DIAERESIS    = U+0x00eb,
  U_e_TREMA        = U+0x00eb,
  U_i_GRAVE        = U+0x00ec,
  U_i_ACUTE,
  U_i_CIRCUMFLEX,
  U_i_DIAERESIS    = U+0x00ef,
  U_i_TREMA        = U+0x00ef,
  U_eth            = U+0x00f0,
  U_n_TILDE,
  U_o_GRAVE        = U+0x00f2,
  U_o_ACUTE,
  U_o_CIRCUMFLEX,
  U_o_TILDE,
  U_o_DIAERESIS    = U+0x00f6,
  U_o_TREMA        = U+0x00f6,
  /* Mathematical operator */
  U_DIVISION_SIGN   = U+0x00f7,
  U_OBELUS          = U+0x00f7, /*<= true? */
  /* Small letters (suit) */
  U_o_STROKE        = U+0x00f8,
  U_o_SLASH         = U+0x00f8,
  U_u_GRAVE         = U+0x00f9,
  U_u_ACUTE,
  U_u_CIRCUMFLEX,
  U_u_DIAERESIS     = U+0x00fc,
  U_u_TREMA         = U+0x00fc,
  U_y_ACUTE         = U+0x00fd,
  U_thorn           = U+0x00fe,
  U_y_DIAERESIS     = U+0x00ff,
  U_y_TREMA         = U+0x00ff,
  
  /* FIXME: */
  
  U_I_TILDE         = U+0x0128,
  
  U_THAI_KHOMUT     = U+0x0e5b,
  
  /* Miscellaneous symols */
  U_MUSIC_FLAT      = U+0x266d,
  U_MUSIC_NATURAL,
  U_MUSIC_SHARP,
  
  /* Musical symbols */
  /* Bars */
  U_MUSICAL_SYMBOL_SINGLE_BARLINE = U+0x1d100,
  U_MUSICAL_SYMBOL_DOUBLE_BARLINE,
  U_MUSICAL_SYMBOL_FINAL_BARLINE,
  U_MUSICAL_SYMBOL_REVERSE_FINAL_BARLINE,
  U_MUSICAL_SYMBOL_DASHED_BARLINE,
  U_MUSICAL_SYMBOL_SHORT_BARLINE,
  /* Codas */
  U_MUSICAL_SYMBOL_LEFT_REPEAT,
  U_MUSICAL_SYMBOL_RIGHT_REPEAT,
  U_MUSICAL_SYMBOL_REPEAT_DOTS,
  U_MUSICAL_SYMBOL_DAL_SEGNO,
  U_MUSICAL_SYMBOL_DA_CAPO,
  U_MUSICAL_SYMBOL_SEGNO,
  U_MUSICAL_SYMBOL_CODA,
  /* Figures repetitions */
  U_MUSICAL_SYMBOL_REPETEAD_FIGURE1,
  U_MUSICAL_SYMBOL_REPETEAD_FIGURE2,
  U_MUSICAL_SYMBOL_REPETEAD_FIGURE3,
  /* Holds and pauses */
  U_MUSICAL_SYMBOL_FERMATA,
  U_MUSICAL_SYMBOL_FERMATA_BELOW,
  U_MUSICAL_SYMBOL_BREATH_MARK,
  U_MUSICAL_SYMBOL_CAESURA,
  /* Staff brackets */
  U_MUSICAL_SYMBOL_BRACE,
  U_MUSICAL_SYMBOL_BRACKET,
  /* Staves */
  U_MUSICAL_SYMBOL_1LINE_STAFF,
  U_MUSICAL_SYMBOL_2LINE_STAFF,
  U_MUSICAL_SYMBOL_3LINE_STAFF,
  U_MUSICAL_SYMBOL_4LINE_STAFF,
  U_MUSICAL_SYMBOL_5LINE_STAFF,
  U_MUSICAL_SYMBOL_6LINE_STAFF,
  /* Tablature */
  U_MUSICAL_SYMBOL_6STRING_FRETBOARD,
  U_MUSICAL_SYMBOL_4STRING_FRETBOARD,
  /* Clefs */
  U_MUSICAL_SYMBOL_G_CLEF,
  U_MUSICAL_SYMBOL_G_CLEF_OTTAVA_ALTA,
  U_MUSICAL_SYMBOL_G_CLEF_OTTAVA_BASSA,
  U_MUSICAL_SYMBOL_C_CLEF,
  U_MUSICAL_SYMBOL_F_CLEF,
  U_MUSICAL_SYMBOL_F_CLEF_OTTAVA_ALTA,
  U_MUSICAL_SYMBOL_F_CLEF_OTTAVA_BASSA,
  U_MUSICAL_SYMBOL_DRUM_CLEF1,
  U_MUSICAL_SYMBOL_DRUM_CLEF2,
  /* Rest */
  U_MUSICAL_SYMBOL_MULTIPLE_MEASURE_REST,
  /* Accidentals */
  U_MUSICAL_SYMBOL_DOUBLE_SHARP,
  U_MUSICAL_SYMBOL_DOUBLE_FLAT,
  U_MUSICAL_SYMBOL_FLAT_UP,
  U_MUSICAL_SYMBOL_FLAT_DOWN,
  U_MUSICAL_SYMBOL_NATURAL_UP,
  U_MUSICAL_SYMBOL_NATURAL_DOWN,
  U_MUSICAL_SYMBOL_SHARP_UP,
  U_MUSICAL_SYMBOL_SHARP_DOWN,
  U_MUSICAL_SYMBOL_QUARTER_TONE_SHARP,
  U_MUSICAL_SYMBOL_QUARTER_TONE_FLAT,
  /* Time signatures */
  U_MUSICAL_SYMBOL_COMMON_TIME,
  U_MUSICAL_SYMBOL_CUT_TIME,
  /* Octaves */
  U_MUSICAL_SYMBOL_OTTAVA_ALTA,
  U_MUSICAL_SYMBOL_OTTAVA_BASSA,
  U_MUSICAL_SYMBOL_QUINDICESIMA_ALTA,
  U_MUSICAL_SYMBOL_QUINDICESIMA_BASSA,
  /* Rests */
  U_MUSICAL_SYMBOL_MULTI_REST, /* see U+0x1d129 U_MUSICAL_SYMBOL_MULTIPLE_MEASURE_REST */
  U_MUSICAL_SYMBOL_WHOLE_REST,
  U_MUSICAL_SYMBOL_HALF_REST,
  U_MUSICAL_SYMBOL_QUARTER_REST,
  U_MUSICAL_SYMBOL_EIGHTH_REST,
  U_MUSICAL_SYMBOL_SIXTEENTH_REST,
  U_MUSICAL_SYMBOL_THIRTY_SECOND_REST = U+0x1d140,
  U_MUSICAL_SYMBOL_THIRTYSECOND_REST  = U+0x1d140,
  U_MUSICAL_SYMBOL_SIXTY_FOURTH_REST  = U+0x1d141,
  U_MUSICAL_SYMBOL_SIXTYFOURTH_REST   = U+0x1d141,
  U_MUSICAL_SYMBOL_ONE_HUNDRED_TWENTY_EIGHTH_REST = U+0x1d142,
  U_MUSICAL_SYMBOL_ONEHUNDREDTWENTYEIGHTH_REST    = U+0x1d142,
  /* Noteheads */
  U_MUSICAL_SYMBOL_X_NOTEHEAD,
  U_MUSICAL_SYMBOL_PLUS_NOTEHEAD,
  U_MUSICAL_SYMBOL_CIRCLE_X_NOTEHEAD,
  U_MUSICAL_SYMBOL_SQUARE_NOTEHEAD_WHITE,
  U_MUSICAL_SYMBOL_SQUARE_NOTEHEAD_BLACK,
  U_MUSICAL_SYMBOL_TRIANGLE_NOTEHEAD_UP_WHITE,
  U_MUSICAL_SYMBOL_TRIANGLE_NOTEHEAD_UP_BLACK,
  U_MUSICAL_SYMBOL_TRIANGLE_NOTEHEAD_LEFT_WHITE,
  U_MUSICAL_SYMBOL_TRIANGLE_NOTEHEAD_LEFT_BLACK,
  U_MUSICAL_SYMBOL_TRIANGLE_NOTEHEAD_RIGHT_WHITE,
  U_MUSICAL_SYMBOL_TRIANGLE_NOTEHEAD_RIGHT_BLACK,
  U_MUSICAL_SYMBOL_TRIANGLE_NOTEHEAD_DOWN_WHITE,
  U_MUSICAL_SYMBOL_TRIANGLE_NOTEHEAD_DOWN_BLACK,
  U_MUSICAL_SYMBOL_TRIANGLE_NOTEHEAD_UP_RIGHT_WHITE,
  U_MUSICAL_SYMBOL_TRIANGLE_NOTEHEAD_UP_RIGHT_BLACK,
  U_MUSICAL_SYMBOL_MOON_NOTEHEAD_WHITE,
  U_MUSICAL_SYMBOL_MOON_NOTEHEAD_BLACK,
  U_MUSICAL_SYMBOL_TRIANGLEROUND_NOTEHEAD_DOWN_WHITE,
  U_MUSICAL_SYMBOL_TRIANGLEROUND_NOTEHEAD_DOWN_BLACK,
  U_MUSICAL_SYMBOL_PARENTHESIS_NOTEHEAD,
  U_MUSICAL_SYMBOL_VOID_NOTEHEAD,
  U_MUSICAL_SYMBOL_NOTEHEAD_BLACK,
  U_MUSICAL_SYMBOL_NULL_NOTEHEAD,
  U_MUSICAL_SYMBOL_CLUSTER_NOTEHEAD_WHITE,
  U_MUSICAL_SYMBOL_CLUSTER_NOTEHEAD_BLACK
  /* TODO: notes & next */
  
};

/* prototypes */
size_t  u8_chlen      (const char* chu8);
long    u8_chval      (const char* chu8);
long    u8_choff      (const char* stru8,
                       long offset);
long    u8_strlen     (const char* stru8);
char*   u8_val2ch     (unsigned long val);
int     u8_valisvalid (long val,
                       size_t len);


#ifdef __cplusplus
}
#endif

#endif /* guard (UTF8_H) */
