/*
 *  Jomar3D, a lobster-running-to-catch game.
 *  Copyright (C) 2006-2008 Jonathan Michalon <studios.chalmion@no-log.org>.
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#ifndef UTILS_H
#define UTILS_H


#include <stdio.h>
#include <dirent.h>


/* if not compiling with GNUC, elide __attribute__ */
#if !defined (__GNUC__) && !defined (__attribute__)
# define __attribute__(x) /* nothing */
#endif


/* Some <math.h> files do not define M_PI... */
#ifndef M_PI
# define M_PI      3.14159265358979323846
#endif

#define raddeg    180./M_PI
#define degrad    M_PI/180.
#define max(a,b)  ((a) >= (b) ? (a) : (b))
#define MYRAND    ((double)rand()/RAND_MAX)
#define MYRANDSYM (MYRAND - 0.5)


/* return 1 if s1 == s2 or 0 elsewhere
 * s1 and s2 must be non-NULL */
#define streq(s1, s2) (!(strcmp (s1, s2)))

/* return the extension of a filename or NULL */
const char* getext  (const char *str);
char*       rmext   (char *const str);

/* scandir doesn't exist in the mingw32 (windows) so make it (simplified) */
int myscandir (const char *dir, struct dirent ***namelist,
               int (*select) (const struct dirent *));

/* malloc and fill a pointer with a string */
char *mystrdup (const char *string);

/* return the string of the time given as argument (in seconds) */
char *time2s (int timeins /* Time in seconds */);

size_t  uintlen  (unsigned long int n, unsigned int base) __attribute__ ((const));
size_t  intlen   (long int n, unsigned int base)          __attribute__ ((const));


#if (DEBUG)
void internal_debugmsg (const char* fmt, ...) __attribute__ ((format(printf,1,2)));
#if defined (__FILE__) && defined (__LINE__) && (defined (_ISOC99_SOURCE) || defined (__GNUC__))
# define debugmsg(x) (fprintf (stdout, "DBG: %s:%d:%s():", __FILE__, __LINE__, __func__), internal_debugmsg x)
#else
# define debugmsg(x) internal_debugmsg x
#endif /* __LINE__ && __FILE__ && __func__ */
#else /* !(DEBUG) */
#define debugmsg(x) /* nothing */
#endif /* (DEBUG) */


#endif /* guard */
